@@ -45,34 +45,21 @@
 // ---------------------------------------------------------------------------
 
 static int
-_e1_rx_hdlcfs(struct e1_ts *ts, const uint8_t *buf, int len)
+_e1_tx_raw(struct e1_ts *ts, uint8_t *buf, int len)
 {
-	int rv, cl, oi;
+		int l;
 
-	oi = 0;
+		l = read(ts->fd, buf, len);
+		/* FIXME: handle underflow */
 
-	while (oi < len) {
-		rv = osmo_isdnhdlc_decode(&ts->hdlc.rx,
-			&buf[oi], len-oi, &cl,
-			ts->hdlc.rx_buf, sizeof(ts->hdlc.rx_buf)
-		);
+		/* If we're not started yet, we 'fake' data until the other side
+		 * send something */
+		if (l < 0 && errno == EAGAIN && !ts->raw.tx_started)
+			return len;
 
-		if (rv > 0) {
-			int bytes_to_write = rv;
-			LOGPTS(ts, DXFR, LOGL_DEBUG, "RX Message: %d [ %s]\n",
-				rv, osmo_hexdump(ts->hdlc.rx_buf, rv));
-			rv = write(ts->fd, ts->hdlc.rx_buf, bytes_to_write);
-			if (rv <= 0)
-				return rv;
-		} else  if (rv < 0 && ts->id == 4) {
-			LOGPTS(ts, DXFR, LOGL_ERROR, "ERR RX: %d %d %d [ %s]\n",
-				rv,oi,cl, osmo_hexdump(buf, len));
-		}
+		ts->raw.tx_started = true;
 
-		oi += cl;
-	}
-
-	return len;
+		return l;
 }
 
 static int
@@ -87,9 +74,9 @@ _e1_tx_hdlcfs(struct e1_ts *ts, uint8_t *buf, int len)
 		if (!ts->hdlc.tx_len) {
 			rv = recv(ts->fd, ts->hdlc.tx_buf, sizeof(ts->hdlc.tx_buf), MSG_TRUNC);
 			if (rv > 0) {
-				if (rv > sizeof(ts->hdlc.tx_buf)) {
+				if (rv > (int)sizeof(ts->hdlc.tx_buf)) {
 					LOGPTS(ts, DXFR, LOGL_ERROR, "Truncated message: Client tried to "
-						"send %d bytes but our buffer is limited to %lu\n",
+						"send %d bytes but our buffer is limited to %zu\n",
 						rv, sizeof(ts->hdlc.tx_buf));
 					rv = sizeof(ts->hdlc.tx_buf);
 				}
@@ -136,8 +123,7 @@ _e1_ts_read(struct e1_ts *ts, uint8_t *buf, size_t len)
 
 	switch (ts->mode) {
 	case E1_TS_MODE_RAW:
-		l = read(ts->fd, buf, len);
-		/* FIXME: handle underflow */
+		l = _e1_tx_raw(ts, buf, len);
 		break;
 	case E1_TS_MODE_HDLCFCS:
 		l = _e1_tx_hdlcfs(ts, buf, len);
@@ -151,7 +137,7 @@ _e1_ts_read(struct e1_ts *ts, uint8_t *buf, size_t len)
 		LOGPTS(ts, DE1D, LOGL_ERROR, "dead socket during read: %s\n",
 			strerror(errno));
 		e1_ts_stop(ts);
-	} else if (l < len) {
+	} else if (l < (int)len) {
 		LOGPTS(ts, DE1D, LOGL_NOTICE, "TS read underflow: We had %zu bytes to read, "
 			"but socket returned only %d\n", len, l);
 	}
@@ -165,8 +151,7 @@ _e1_line_mux_out_channelized(struct e1_line *line, uint8_t *buf, int fts)
 	OSMO_ASSERT(line->mode == E1_LINE_MODE_CHANNELIZED);
 
 	/* Scan timeslots */
-	for (int tsn=1; tsn<32; tsn++)
-	{
+	for (int tsn = 1; tsn < 32; tsn++) {
 		struct e1_ts *ts = &line->ts[tsn];
 		uint8_t buf_ts[fts];
 		int l;
@@ -178,7 +163,7 @@ _e1_line_mux_out_channelized(struct e1_line *line, uint8_t *buf, int fts)
 		if (l <= 0)
 			continue;
 
-		for (int i=0; i<l; i++)
+		for (int i = 0; i < l; i++)
 			buf[tsn+(i*32)] = buf_ts[i];
 	}
 }
@@ -226,10 +211,15 @@ e1_line_mux_out(struct e1_line *line, uint8_t *buf, int fts)
 	case E1_LINE_MODE_SUPERCHANNEL:
 		_e1_line_mux_out_superchan(line, buf, fts);
 		break;
+	case E1_LINE_MODE_E1OIP:
+		e1oip_line_mux_out(line, buf, fts);
+		break;
 	default:
 		OSMO_ASSERT(0);
 	}
 
+	line_ctr_add(line, LINE_CTR_FRAMES_MUXED_E1T, fts);
+
 	return tsz;
 }
 
@@ -275,6 +265,37 @@ _e1_rx_raw(struct e1_ts *ts, const uint8_t *buf, unsigned int len)
 	return appended;
 }
 
+static int
+_e1_rx_hdlcfs(struct e1_ts *ts, const uint8_t *buf, int len)
+{
+	int rv, cl, oi;
+
+	oi = 0;
+
+	while (oi < len) {
+		rv = osmo_isdnhdlc_decode(&ts->hdlc.rx,
+			&buf[oi], len-oi, &cl,
+			ts->hdlc.rx_buf, sizeof(ts->hdlc.rx_buf)
+		);
+
+		if (rv > 0) {
+			int bytes_to_write = rv;
+			LOGPTS(ts, DXFR, LOGL_DEBUG, "RX Message: %d [ %s]\n",
+				rv, osmo_hexdump(ts->hdlc.rx_buf, rv));
+			rv = write(ts->fd, ts->hdlc.rx_buf, bytes_to_write);
+			if (rv <= 0)
+				return rv;
+		} else  if (rv < 0 && ts->id == 4) {
+			LOGPTS(ts, DXFR, LOGL_ERROR, "ERR RX: %d %d %d [ %s]\n",
+				rv, oi, cl, osmo_hexdump(buf, len));
+		}
+
+		oi += cl;
+	}
+
+	return len;
+}
+
 /* write data to a timeslot (hardware -> application direction) */
 static int
 _e1_ts_write(struct e1_ts *ts, const uint8_t *buf, size_t len)
@@ -297,7 +318,7 @@ _e1_ts_write(struct e1_ts *ts, const uint8_t *buf, size_t len)
 		LOGPTS(ts, DE1D, LOGL_ERROR, "dead socket during write: %s\n",
 			strerror(errno));
 		e1_ts_stop(ts);
-	} else if (rv < len) {
+	} else if (rv < (int)len) {
 		LOGPTS(ts, DE1D, LOGL_NOTICE, "TS write overflow: We had %zu bytes to send, "
 			"but write returned only %d\n", len, rv);
 	}
@@ -305,6 +326,27 @@ _e1_ts_write(struct e1_ts *ts, const uint8_t *buf, size_t len)
 	return rv;
 }
 
+static int
+_e1_line_demux_in_channelized(struct e1_line *line, const uint8_t *buf, int ftr)
+{
+	OSMO_ASSERT(line->mode == E1_LINE_MODE_CHANNELIZED);
+
+	for (int tsn = 1; tsn < 32; tsn++) {
+		struct e1_ts *ts = &line->ts[tsn];
+		uint8_t buf_ts[ftr];
+
+		if (ts->mode == E1_TS_MODE_OFF)
+			continue;
+
+		for (int i = 0; i < ftr; i++)
+			buf_ts[i] = buf[tsn+(i*32)];
+
+		_e1_ts_write(ts, buf_ts, ftr);
+	}
+
+	return 0;
+}
+
 static int
 _e1_line_demux_in_superchan(struct e1_line *line, const uint8_t *buf, int ftr)
 {
@@ -326,28 +368,6 @@ _e1_line_demux_in_superchan(struct e1_line *line, const uint8_t *buf, int ftr)
 	return 0;
 }
 
-static int
-_e1_line_demux_in_channelized(struct e1_line *line, const uint8_t *buf, int ftr)
-{
-	OSMO_ASSERT(line->mode == E1_LINE_MODE_CHANNELIZED);
-
-	for (int tsn=1; tsn<32; tsn++)
-	{
-		struct e1_ts *ts = &line->ts[tsn];
-		uint8_t buf_ts[ftr];
-
-		if (ts->mode == E1_TS_MODE_OFF)
-			continue;
-
-		for (int i=0; i<ftr; i++)
-			buf_ts[i] = buf[tsn+(i*32)];
-
-		_e1_ts_write(ts, buf_ts, ftr);
-	}
-
-	return 0;
-}
-
 static void
 _e1_line_demux_in_ts0(struct e1_line *line, const uint8_t *buf, int ftr, uint8_t frame_base)
 {
@@ -359,8 +379,10 @@ _e1_line_demux_in_ts0(struct e1_line *line, const uint8_t *buf, int ftr, uint8_t
 
 		/* A bit is present in each odd frame */
 		if (frame_nr % 2) {
-			if (frame[0] & 0x20)
+			if (frame[0] & 0x20) {
 				line->ts0.cur_errmask |= E1L_TS0_RX_ALARM;
+				line_ctr_add(line, LINE_CTR_RX_REMOTE_A, 1);
+			}
 		}
 
 		/* E bits are present in frame 13 + 15 */
@@ -368,8 +390,10 @@ _e1_line_demux_in_ts0(struct e1_line *line, const uint8_t *buf, int ftr, uint8_t
 			line->ts0.e_bits = frame[0] & 0x80 ? 2 : 0;
 		if (frame_nr == 15) {
 			line->ts0.e_bits |= frame[0] & 0x80 ? 1 : 0;
-			if (line->ts0.e_bits != 3)
+			if (line->ts0.e_bits != 3) {
 				line->ts0.cur_errmask |= E1L_TS0_RX_CRC4_ERR;
+				line_ctr_add(line, LINE_CTR_RX_REMOTE_E, 1);
+			}
 		}
 		/* cur_errmask is being cleared once per second via line->ts0.timer */
 	}
@@ -399,11 +423,15 @@ e1_line_demux_in(struct e1_line *line, const uint8_t *buf, int size, int frame_b
 	if (frame_base >= 0)
 		_e1_line_demux_in_ts0(line, buf, ftr, frame_base);
 
+	line_ctr_add(line, LINE_CTR_FRAMES_DEMUXED_E1O, ftr);
+
 	switch (line->mode) {
 	case E1_LINE_MODE_CHANNELIZED:
 		return _e1_line_demux_in_channelized(line, buf, ftr);
 	case E1_LINE_MODE_SUPERCHANNEL:
 		return _e1_line_demux_in_superchan(line, buf, ftr);
+	case E1_LINE_MODE_E1OIP:
+		return e1oip_line_demux_in(line, buf, ftr);
 	default:
 		OSMO_ASSERT(0);
 	}
