@@ -8,6 +8,8 @@ extern struct cmd_element cfg_account_mode_cmd;
 extern struct cmd_element cfg_account_ice1_serno_cmd;
 extern struct cmd_element cfg_account_ice1_line_cmd;
 extern struct cmd_element cfg_account_batching_factor_cmd;
+extern struct cmd_element cfg_account_force_all_ts_cmd;
+extern struct cmd_element cfg_account_no_force_all_ts_cmd;
 extern struct cmd_element cfg_account_prefill_frame_count_cmd;
 extern struct cmd_element cfg_account_trunkdev_name_cmd;
 extern struct cmd_element cfg_account_trunkdev_line_cmd;
