@@ -2,6 +2,7 @@
  * usb.c
  *
  * (C) 2019 by Sylvain Munaut <tnt@246tNt.com>
+ * (C) 2022 by Harald Welte <laforge@gnumonks.org>
  *
  * All Rights Reserved
  *
@@ -42,6 +43,7 @@
 
 #define USB_VID		0x1d50
 #define USB_PID		0x6145
+#define USB_PID_TRACER	0x6151
 
 libusb_context *g_usb = NULL;
 
@@ -635,8 +637,8 @@ e1_usb_ctrl_set_gpsdo_mode(struct e1_intf *intf, enum ice1usb_gpsdo_mode gpsdo_m
 {
 	const uint16_t bmReqType = LIBUSB_RECIPIENT_INTERFACE | LIBUSB_REQUEST_TYPE_VENDOR |
 				   LIBUSB_ENDPOINT_OUT;
-	return _e1_usb_intf_send_ctrl(intf, bmReqType, ICE1USB_INTF_SET_GPSDO_MODE, 0,
-				      (uint8_t *)&gpsdo_mode, sizeof(gpsdo_mode));
+	return _e1_usb_intf_send_ctrl(intf, bmReqType, ICE1USB_INTF_SET_GPSDO_MODE, gpsdo_mode,
+				      NULL, 0);
 }
 
 int
@@ -645,7 +647,7 @@ e1_usb_ctrl_set_gpsdo_tune(struct e1_intf *intf, const struct e1usb_gpsdo_tune *
 	const uint16_t bmReqType = LIBUSB_RECIPIENT_INTERFACE | LIBUSB_REQUEST_TYPE_VENDOR |
 				   LIBUSB_ENDPOINT_OUT;
 	return _e1_usb_intf_send_ctrl(intf, bmReqType, ICE1USB_INTF_SET_GPSDO_TUNE, 0,
-				      (uint8_t *)gpsdo_tune, sizeof(gpsdo_tune));
+				      (uint8_t *)gpsdo_tune, sizeof(*gpsdo_tune));
 }
 
 int
@@ -692,6 +694,12 @@ e1_usb_intf_gpsdo_state_string(char *buf, size_t len, const struct e1_intf *intf
 
 	OSMO_ASSERT(intf->drv == E1_DRIVER_USB);
 
+	if (!id) {
+		/* This can happen for statically configured devices (config/vty) which are not
+		 * currently present */
+		return snprintf(buf, len, "unknown");
+	}
+
 	return snprintf(buf, len, "mode=%s, fix=%s, state=%s antenna=%s, tune=%u/%u, freq_est=%u",
 			get_value_string(ice1usb_gpsdo_mode_str, last_st->mode),
 			last_st->valid_fix ? "TRUE" : "FALSE",
@@ -706,14 +714,29 @@ _e1_usb_intf_gpsdo_status_cb(struct e1_intf *intf, const uint8_t *data, size_t l
 {
 	struct e1_usb_intf_data *id = intf->drv_data;
 	struct e1usb_gpsdo_status *last_st = &id->gpsdo.last_status;
-	const struct e1usb_gpsdo_status *st;
+	struct e1usb_gpsdo_status _st, *st = &_st;
 	struct e1_line *line;
 
 	if (len < sizeof(*st)) {
-		LOGPIF(intf, DE1D, LOGL_ERROR, "GPSDO status %zu < %zu!\n", len, sizeof(*st));
-		return;
+		/*
+		 * Because some fields can be added to the structure by newer
+		 * firmware revisions, this means we can potentially get a shorter
+		 * struct than what we asked for. We simply set those fields to
+		 * zero.
+		 *
+		 * The opposite case (newer firmware than e1d) means the structure
+		 * could be larger, but because we limit the wLength to the struct
+		 * we know, we can't receive a larger one and the new fields are
+		 * just ignored by this e1d version
+		 */
+		LOGPIF(intf, DE1D, LOGL_DEBUG,
+			"GPSDO status %zu < %zu ! Firmware probably outdated. "
+			"Some values will be zeroed\n",
+			len, sizeof(*st));
 	}
-	st = (const struct e1usb_gpsdo_status *) data;
+
+	memset(st, 0x00, sizeof(*st));
+	memcpy(st, data, len);
 
 	if (st->state != last_st->state) {
 		LOGPIF(intf, DE1D, LOGL_NOTICE, "GPSDO state change: %s -> %s\n",
@@ -750,6 +773,7 @@ _e1_usb_intf_gpsdo_status_cb(struct e1_intf *intf, const uint8_t *data, size_t l
 		line_stat_set(line, LINE_GPSDO_TUNE_COARSE, libusb_le16_to_cpu(st->tune.coarse));
 		line_stat_set(line, LINE_GPSDO_TUNE_FINE, libusb_le16_to_cpu(st->tune.fine));
 		line_stat_set(line, LINE_GPSDO_FREQ_EST, osmo_load32le(&st->freq_est));
+		line_stat_set(line, LINE_GPSDO_ERR_ACC, (int16_t)libusb_le16_to_cpu(st->err_acc));
 	}
 
 	/* update our state */
@@ -773,6 +797,17 @@ _e1_usb_gpsdo_init(struct e1_intf *intf)
 {
 	struct e1_usb_intf_data *id = intf->drv_data;
 
+	if (intf->usb.gpsdo.manual) {
+		struct e1usb_gpsdo_tune tune = {
+			.coarse = intf->usb.gpsdo.coarse,
+			.fine   = intf->usb.gpsdo.fine,
+		};
+		e1_usb_ctrl_set_gpsdo_mode(intf, ICE1USB_GPSDO_MODE_DISABLED);
+		e1_usb_ctrl_set_gpsdo_tune(intf, &tune);
+	} else {
+		e1_usb_ctrl_set_gpsdo_mode(intf, ICE1USB_GPSDO_MODE_AUTO);
+	}
+
 	osmo_timer_setup(&id->gpsdo.poll_timer, &_e1_usb_gpsdo_poll_cb, intf);
 	osmo_timer_schedule(&id->gpsdo.poll_timer, 1, 0);
 }
@@ -782,7 +817,7 @@ _e1_usb_gpsdo_init(struct e1_intf *intf)
 // ---------------------------------------------------------------------------
 
 static int
-_e1_usb_open_device(struct e1_daemon *e1d, struct libusb_device *dev)
+_e1_usb_open_device(struct e1_daemon *e1d, struct libusb_device *dev, bool is_tracer)
 {
 	struct e1_intf *intf;
 	struct e1_line *line;
@@ -796,6 +831,10 @@ _e1_usb_open_device(struct e1_daemon *e1d, struct libusb_device *dev)
 	libusb_device_handle *devh;
 	int line_nr = 0;
 	int i, j, ret;
+	const char *hwname = "icE1usb";
+
+	if (is_tracer)
+		hwname = "e1-tracer";
 
 	ret = libusb_open(dev, &devh);
 	if (ret) {
@@ -824,7 +863,7 @@ _e1_usb_open_device(struct e1_daemon *e1d, struct libusb_device *dev)
 	/* try to find the matching interface config created by the vty */
 	intf = e1d_find_intf_by_usb_serial(e1d, serial_str);
 	if (intf) {
-		LOGP(DE1D, LOGL_INFO, "Configuration for icE1usb serial '%s' found\n", serial_str);
+		LOGP(DE1D, LOGL_INFO, "Configuration for %s serial '%s' found\n", hwname, serial_str);
 		auto_create_lines = false;
 		if (intf->drv_data) {
 			LOGP(DE1D, LOGL_ERROR, "New device with serial '%s', but E1 interface %u busy\n",
@@ -836,8 +875,8 @@ _e1_usb_open_device(struct e1_daemon *e1d, struct libusb_device *dev)
 		intf_data->devh = devh;
 		intf->drv_data = intf_data;
 	} else {
-		LOGP(DE1D, LOGL_NOTICE, "No configuration for icE1usb serial '%s' found, "
-		     "auto-generating it\n", serial_str);
+		LOGP(DE1D, LOGL_NOTICE, "No configuration for %s serial '%s' found, "
+		     "auto-generating it\n", hwname, serial_str);
 		auto_create_lines = true;
 		intf_data = talloc_zero(e1d->ctx, struct e1_usb_intf_data);
 		intf_data->devh = devh;
@@ -846,11 +885,20 @@ _e1_usb_open_device(struct e1_daemon *e1d, struct libusb_device *dev)
 		osmo_talloc_replace_string(intf, &intf->usb.serial_str, serial_str);
 	}
 
+	/* we have prior knowledge that the e1-tracer firmware configuration 2 is the e1d compatible mode. */
+	if (is_tracer) {
+		if (libusb_set_configuration(devh, 2) != LIBUSB_SUCCESS) {
+			LOGP(DE1D, LOGL_ERROR, "Cannot set configuration 2 of e1-tracer device. Maybe too old firmware?\n");
+			libusb_close(devh);
+			return -EIO;
+		}
+	}
+
 	INIT_LLIST_HEAD(&intf_data->ctrl_inprogress);
 
 	ret = libusb_get_active_config_descriptor(dev, &cd);
 	if (ret) {
-		LOGP(DE1D, LOGL_ERROR, "Failed to talk to usb device: %s\n", libusb_strerror(ret));
+		LOGP(DE1D, LOGL_ERROR, "Failed to talk to %s usb device: %s\n", hwname, libusb_strerror(ret));
 		intf_data->devh = NULL;
 		talloc_free(intf_data);
 		if (auto_create_lines)
@@ -865,8 +913,13 @@ _e1_usb_open_device(struct e1_daemon *e1d, struct libusb_device *dev)
 			continue;
 
 		id = &cd->interface[i].altsetting[1];
-		if ((id->bInterfaceClass != 0xff) || (id->bInterfaceSubClass != 0xe1) || (id->bNumEndpoints < 3))
-			continue;
+		if (is_tracer) {
+			if ((id->bInterfaceClass != 0xff) || (id->bInterfaceSubClass != 0xe1) || (id->bNumEndpoints < 1))
+				continue;
+		} else {
+			if ((id->bInterfaceClass != 0xff) || (id->bInterfaceSubClass != 0xe1) || (id->bNumEndpoints < 3))
+				continue;
+		}
 
 		line = e1_intf_find_line(intf, line_nr);
 		if (line) {
@@ -877,21 +930,6 @@ _e1_usb_open_device(struct e1_daemon *e1d, struct libusb_device *dev)
 			}
 		}
 
-		/* Get interface and set it up */
-		ret = libusb_claim_interface(devh, id->bInterfaceNumber);
-		if (ret) {
-			LOGP(DE1D, LOGL_ERROR, "Failed to claim interface %d:%s\n", id->bInterfaceNumber,
-			     libusb_strerror(ret));
-			goto next_interface;
-		}
-
-		ret = libusb_set_interface_alt_setting(devh, id->bInterfaceNumber, 1);
-		if (ret) {
-			LOGP(DE1D, LOGL_ERROR, "Failed to set interface %d altsetting:%s\n", id->bInterfaceNumber,
-			     libusb_strerror(ret));
-			goto next_interface;
-		}
-
 		/* Setup driver data and find endpoints */
 		line_data = talloc_zero(e1d->ctx, struct e1_usb_line_data);
 
@@ -921,9 +959,16 @@ _e1_usb_open_device(struct e1_daemon *e1d, struct libusb_device *dev)
 			}
 		}
 
-		if (!line_data->ep_in || !line_data->ep_out || !line_data->ep_fb || !line_data->pkt_size) {
-			LOGP(DE1D, LOGL_ERROR, "Failed to use interface %d\n", id->bInterfaceNumber);
-			goto next_interface;
+		if (is_tracer) {
+			if (!line_data->ep_in || !line_data->pkt_size) {
+				LOGP(DE1D, LOGL_ERROR, "Failed to use interface %d\n", id->bInterfaceNumber);
+				goto next_interface;
+			}
+		} else {
+			if (!line_data->ep_in || !line_data->ep_out || !line_data->ep_fb || !line_data->pkt_size) {
+				LOGP(DE1D, LOGL_ERROR, "Failed to use interface %d\n", id->bInterfaceNumber);
+				goto next_interface;
+			}
 		}
 
 		if (!line) {
@@ -938,13 +983,34 @@ _e1_usb_open_device(struct e1_daemon *e1d, struct libusb_device *dev)
 			line->drv_data = line_data;
 		}
 
-		line_data->flow_in  = e1uf_create(line, e1_usb_xfer_in,  line_data->ep_in,  4, line_data->pkt_size, 4);
-		line_data->flow_out = e1uf_create(line, e1_usb_xfer_out, line_data->ep_out, 4, line_data->pkt_size, 4);
-		line_data->flow_fb  = e1uf_create(line, e1_usb_xfer_fb,  line_data->ep_fb,  2, 3, 1);
+		/* Get interface and set it up */
+		ret = libusb_claim_interface(devh, id->bInterfaceNumber);
+		if (ret) {
+			LOGP(DE1D, LOGL_ERROR, "Failed to claim interface %d:%s\n", id->bInterfaceNumber,
+			     libusb_strerror(ret));
+			goto next_interface;
+		}
 
+		ret = libusb_set_interface_alt_setting(devh, id->bInterfaceNumber, 1);
+		if (ret) {
+			LOGP(DE1D, LOGL_ERROR, "Failed to set interface %d altsetting:%s\n", id->bInterfaceNumber,
+			     libusb_strerror(ret));
+			goto next_interface;
+		}
+
+		/* Create data flows and start the line */
+
+		/* all supported devices have an IN endpoint */
+		line_data->flow_in  = e1uf_create(line, e1_usb_xfer_in,  line_data->ep_in,  4, line_data->pkt_size, 4);
 		e1uf_start(line_data->flow_in);
-		e1uf_start(line_data->flow_out);
-		e1uf_start(line_data->flow_fb);
+
+		/* e1-tracer has no OUT or FEEDBACK endpoint */
+		if (!is_tracer) {
+			line_data->flow_out = e1uf_create(line, e1_usb_xfer_out, line_data->ep_out, 4, line_data->pkt_size, 4);
+			e1uf_start(line_data->flow_out);
+			line_data->flow_fb  = e1uf_create(line, e1_usb_xfer_fb,  line_data->ep_fb,  2, 3, 1);
+			e1uf_start(line_data->flow_fb);
+		}
 
 		if (line_data->ep_int)
 			resubmit_irq(line);
@@ -1000,10 +1066,19 @@ e1_usb_probe(struct e1_daemon *e1d)
 		if (ret)
 			continue;
 
-		if ((desc.idVendor != USB_VID) || (desc.idProduct != USB_PID))
+		if (desc.idVendor != USB_VID)
 			continue;
 
-		_e1_usb_open_device(e1d, dev_list[i]);
+		switch (desc.idProduct) {
+		case USB_PID:
+			_e1_usb_open_device(e1d, dev_list[i], false);
+			break;
+		case USB_PID_TRACER:
+			_e1_usb_open_device(e1d, dev_list[i], true);
+			break;
+		default:
+			continue;
+		}
 	}
 
 	libusb_free_device_list(dev_list, 1);
